//**********************************************************************
//  DIVE Laboratories, Inc.
//  Copyright(c) 1995
//  All rights reserved.
//  FILE:   PLATTER.HPP
//
//  DESCRIPTION
//  This file provides the function prototypes for the platterClass
//  that is the driver for the standard window platter.
//
//  Author: M. Doucet
//
//  Modification History:
//  7/3/95	Created
//
//**********************************************************************
#ifndef _PLATTER_HPP
#define _PLATTER_HPP

#include "sensor.hpp"
#include "style.hpp"
#include "vector.hpp"
#include "position.hpp"


class platterClass : public sensorClass, public styleClass {

private:

	V3  mRotations;
	V3  mDeltas;
	vectorClass vec, xRef, yRef;
	positionClass  pos;
	Vres LOS_VAL;
	float wW, wH, mX, mY, minP, maxP;
	void *hostWnd;

	void doUpdate();

public:

	// Button down flags
	int rightBD, leftBD, middleBD;

	void style(void);
#ifdef WIN32
	void mouseEvent(mouseEventEnum event, I2 point, void *wnd);
#endif
	void mouseMove(float width, float height, float platterX, float platterY);
	void leftButtonDown(float width, float height, float platterX, float platterY);
	void rightButtonDown(float width, float height, float platterX, float platterY);
	void leftButtonUp();
	void rightButtonUp();
	void setMinMaxPitch(float minPitch, float maxPitch);
	void setRadius(float radius);
	void setAngles(F2 angles);

	platterClass(void                   *wnd,
	             sensorAngleEnumType    sa=ABSOLUTE_ANGLES,
	             float                  initialRadius = 0.0,
	             F2                     initialAngles = zeroF2);

	~platterClass();

};

#endif
