//**********************************************************************
//  DIVE Laboratories, Inc.
//  Copyright(c) 1995
//  All rights reserved.
//  FILE:   VRTYPES.HPP
//
//  DESCRIPTION
//  This file provides the type definitions for Amber's general types
//
//  Author: M. Doucet
//
//  Modification History:
//  3/22/95	Created
//
//**********************************************************************
#ifndef VRTYPES_HPP
#define VRTYPES_HPP

#include <math.h>
#include <string.h>

class lawClass;
class kinematicClass;

typedef  void   (*ACTIONFUNCPTR)(void *obj); 
typedef  void  (*LAWFUNCPTR)(lawClass *obj);
typedef  void  (*GEOLAWFUNCPTR)(kinematicClass *obj);


typedef enum {_UNKNOWN_FILE,
              _NFF_FILE, _VRML_FILE, _DXF_FILE, _FLT_FILE, _OBJ_FILE, _3DS_FILE,
              _TGA_FILE, _BMP_FILE, _JPEG_FILE, _TIFF_FILE,
				  _MIDI_FILE, _WAV_FILE} fileEnumType;

typedef enum {_OPAQUE, _TRANSLUCENT} opacityEnumType;
typedef enum {_STATIC, _DYNAMIC}     entityEnumType;
typedef enum {_POINTS, _LINES, _TRIS, _POLYS} polyAsEnum;
typedef enum {_LINE_STRIP, _LINE_LOOP} lineAsEnum;
typedef enum {_FILL, _NOFILL}                 polyFillEnum;
typedef enum {_FLAT, _SHADED}                 polyShadingEnum;
typedef enum {_LOCAL, _WORLD} frameEnum;

typedef enum {_NOCOMM, _SOCKET, _SCRAMNET} 	renderCommEnum;
typedef enum {_NOSL, _SELF, _REMOTE} 			selfRemoteEnum;
typedef enum {_FINE, _MEDIUM, _COARSE}			resolutionEnum;

typedef enum {_LEFT,  _RIGHT,  _XCENTER} pivotXenum;
typedef enum {_TOP,   _BOTTOM, _YCENTER} pivotYenum;
typedef enum {_FRONT, _BACK,   _ZCENTER} pivotZenum;

typedef enum {_LINEAR, _EXP, _EXP2} fogModeTypeEnum;

typedef void   (*VOIDFUNCPTRV)(void);
typedef int    (*INTFUNCPTRV)(void);
typedef float  (*FLOATFUNCPTRV)(void);
typedef double (*DOUBLEFUNCPTRV)(void);

typedef unsigned char UC2[2];
typedef char    C2[2];
typedef int     I2[2];
typedef float   F2[2];
typedef double  D2[2];

typedef unsigned char UC3[3];
typedef char    C3[3];
typedef int     I3[3];
typedef float   F3[3];
typedef double  D3[3];

typedef unsigned char UC4[4];
typedef char    C4[4];
typedef int     I4[4];
typedef float   F4[4];
typedef double  D4[4];

#ifdef AMBER_3DR
typedef float   M16[4][4];
#endif
#ifdef AMBER_OPENGL
typedef float   M16[16];
#endif

typedef unsigned char flagType;

#define V2_set(x, v0, v1)	( x[0]=v0,\
									  x[1]=v1)
#define V3_set(x, v0, v1, v2)	( x[0]=v0,\
									  	  x[1]=v1,\
									  	  x[2]=v2)

#define V4_set(x, v0, v1, v2, v3)( x[0]=v0,\
									  	     x[1]=v1,\
									  	     x[2]=v2,\
									  	     x[3]=v3)

#define I2_set(x, v0, v1) V2_set(x, v0, v1)
#define I3_set(x, v0, v1, v2) V3_set(x, v0, v1, v2)
#define I4_set(x, v0, v1, v2, v3) V4_set(x, v0, v1, v2, v3)
#define F2_set(x, v0, v1) V2_set(x, v0, v1)
#define F3_set(x, v0, v1, v2) V3_set(x, v0, v1, v2)
#define F4_set(x, v0, v1, v2, v3) V4_set(x, v0, v1, v2, v3)
#define D2_set(x, v0, v1) V2_set(x, v0, v1)
#define D3_set(x, v0, v1, v2) V3_set(x, v0, v1, v2)
#define D4_set(x, v0, v1, v2, v3) V4_set(x, v0, v1, v2, v3)

#define V2_init(x)	x[0]=x[1]=0;
#define V3_init(x)	x[0]=x[1]=x[2]=0;
#define V4_init(x)	x[0]=x[1]=x[2]=x[3]=0;

#define UC2_init(x) V2_init(x)
#define UC3_init(x) V3_init(x)
#define UC4_init(x) V4_init(x)
#define C2_init(x) V2_init(x)
#define C3_init(x) V3_init(x)
#define C4_init(x) V4_init(x)
#define I2_init(x) V2_init(x)
#define I3_init(x) V3_init(x)
#define I4_init(x) V4_init(x)
#define F2_init(x) V2_init(x)
#define F3_init(x) V3_init(x)
#define F4_init(x) V4_init(x)
#define D2_init(x) V2_init(x)
#define D3_init(x) V3_init(x)
#define D4_init(x) V4_init(x)

#define V2_equal(x,y)	(x[0]==y[0] && x[1]==y[1])
#define V3_equal(x,y)	(x[0]==y[0] && x[1]==y[1] && x[2]==y[2])
#define V4_equal(x,y)	(x[0]==y[0] && x[1]==y[1] && x[2]==y[2] && x[3]==y[3])

#define UC2_equal(x,y) V2_equal(x,y)
#define UC3_equal(x,y) V3_equal(x,y)
#define UC4_equal(x,y) V4_equal(x,y)
#define C2_equal(x,y) V2_equal(x,y)
#define C3_equal(x,y) V3_equal(x,y)
#define C4_equal(x,y) V4_equal(x,y)
#define I2_equal(x,y) V2_equal(x,y)
#define I3_equal(x,y) V3_equal(x,y)
#define I4_equal(x,y) V4_equal(x,y)
#define F2_equal(x,y) V2_equal(x,y)
#define F3_equal(x,y) V3_equal(x,y)
#define F4_equal(x,y) V4_equal(x,y)
#define D2_equal(x,y) V2_equal(x,y)
#define D3_equal(x,y) V3_equal(x,y)
#define D4_equal(x,y) V4_equal(x,y)

#define UC2_copy(dest,src)	memcpy((void *)dest, (void *)src, sizeof(UC2))
#define UC3_copy(dest,src)	memcpy((void *)dest, (void *)src, sizeof(UC3))
#define UC4_copy(dest,src)	memcpy((void *)dest, (void *)src, sizeof(UC4))
#define C2_copy(dest,src)	memcpy((void *)dest, (void *)src, sizeof(C2))
#define C3_copy(dest,src)	memcpy((void *)dest, (void *)src, sizeof(C3))
#define C4_copy(dest,src)	memcpy((void *)dest, (void *)src, sizeof(C4))
#define I2_copy(dest,src)	memcpy((void *)dest, (void *)src, sizeof(I2))
#define I3_copy(dest,src)	memcpy((void *)dest, (void *)src, sizeof(I3))
#define I4_copy(dest,src)	memcpy((void *)dest, (void *)src, sizeof(I4))
#define F2_copy(dest,src)	memcpy((void *)dest, (void *)src, sizeof(F2))
#define F3_copy(dest,src)	memcpy((void *)dest, (void *)src, sizeof(F3))
#define F4_copy(dest,src)	memcpy((void *)dest, (void *)src, sizeof(F4))
#define D2_copy(dest,src)	memcpy((void *)dest, (void *)src, sizeof(D2))
#define D3_copy(dest,src)	memcpy((void *)dest, (void *)src, sizeof(D3))
#define D4_copy(dest,src)	memcpy((void *)dest, (void *)src, sizeof(D4))

#define V2_dot(x,y) (x[0]*y[0] + x[1]*y[1])
#define V3_dot(x,y) (x[0]*y[0] + x[1]*y[1] + x[2]*y[2])
#define V4_dot(x,y) (x[0]*y[0] + x[1]*y[1] + x[2]*y[2] + x[3]*y[3])

#define I2_dot(x,y) V2_dot(x,y)
#define I3_dot(x,y) V3_dot(x,y)
#define I4_dot(x,y) V4_dot(x,y)
#define F2_dot(x,y) V2_dot(x,y)
#define F3_dot(x,y) V3_dot(x,y)
#define F4_dot(x,y) V4_dot(x,y)
#define D2_dot(x,y) V2_dot(x,y)
#define D3_dot(x,y) V3_dot(x,y)
#define D4_dot(x,y) V4_dot(x,y)

#define V2_add(x,y,z) ( (z)[0] = (x)[0] + (y)[0],\
								(z)[1] = (x)[1] + (y)[1])
#define V3_add(x,y,z) ( (z)[0] = (x)[0] + (y)[0],\
								(z)[1] = (x)[1] + (y)[1],\
								(z)[2] = (x)[2] + (y)[2])
#define V4_add(x,y,z) ( (z)[0] = (x)[0] + (y)[0],\
								(z)[1] = (x)[1] + (y)[1],\
								(z)[2] = (x)[2] + (y)[2],\
								(z)[3] = (x)[3] + (y)[3])

#define I2_add(x,y,z) V2_add(x,y,z)
#define I3_add(x,y,z) V3_add(x,y,z)
#define I4_add(x,y,z) V4_add(x,y,z)
#define F2_add(x,y,z) V2_add(x,y,z)
#define F3_add(x,y,z) V3_add(x,y,z)
#define F4_add(x,y,z) V4_add(x,y,z)
#define D2_add(x,y,z) V2_add(x,y,z)
#define D3_add(x,y,z) V3_add(x,y,z)
#define D4_add(x,y,z) V4_add(x,y,z)

#define V2_sub(x,y,z) ( (z)[0] = (x)[0] - (y)[0],\
								(z)[1] = (x)[1] - (y)[1])
#define V3_sub(x,y,z) ( (z)[0] = (x)[0] - (y)[0],\
								(z)[1] = (x)[1] - (y)[1],\
								(z)[2] = (x)[2] - (y)[2])
#define V4_sub(x,y,z) ( (z)[0] = (x)[0] - (y)[0],\
								(z)[1] = (x)[1] - (y)[1],\
								(z)[2] = (x)[2] - (y)[2],\
								(z)[3] = (x)[3] - (y)[3])

#define I2_sub(x,y,z) V2_sub(x,y,z)
#define I3_sub(x,y,z) V3_sub(x,y,z)
#define I4_sub(x,y,z) V4_sub(x,y,z)
#define F2_sub(x,y,z) V2_sub(x,y,z)
#define F3_sub(x,y,z) V3_sub(x,y,z)
#define F4_sub(x,y,z) V4_sub(x,y,z)
#define D2_sub(x,y,z) V2_sub(x,y,z)
#define D3_sub(x,y,z) V3_sub(x,y,z)
#define D4_sub(x,y,z) V4_sub(x,y,z)

#define V2_multk(x,k) ((x)[0] *= k, (x)[1] *= k)
#define V3_multk(x,k) ((x)[0] *= k, (x)[1] *= k, (x)[2] *= k)
#define V4_multk(x,k) ((x)[0] *= k, (x)[1] *= k, (x)[2] *= k, (x)[3] *= k)

#define I2_multk(x,k) V2_multk(x,k)
#define I3_multk(x,k) V3_multk(x,k)
#define I4_multk(x,k) V4_multk(x,k)
#define F2_multk(x,k) V2_multk(x,k)
#define F3_multk(x,k) V3_multk(x,k)
#define F4_multk(x,k) V4_multk(x,k)
#define D2_multk(x,k) V2_multk(x,k)
#define D3_multk(x,k) V3_multk(x,k)
#define D4_multk(x,k) V4_multk(x,k)

#define V2_divk(x,k) ((x)[0] /= k, (x)[1] /= k)
#define V3_divk(x,k) ((x)[0] /= k, (x)[1] /= k, (x)[2] /= k)
#define V4_divk(x,k) ((x)[0] /= k, (x)[1] /= k, (x)[2] /= k, (x)[3] /= k)

#define I2_divk(x,k) V2_divk(x,k)
#define I3_divk(x,k) V3_divk(x,k)
#define I4_divk(x,k) V4_divk(x,k)
#define F2_divk(x,k) V2_divk(x,k)
#define F3_divk(x,k) V3_divk(x,k)
#define F4_divk(x,k) V4_divk(x,k)
#define D2_divk(x,k) V2_divk(x,k)
#define D3_divk(x,k) V3_divk(x,k)
#define D4_divk(x,k) V4_divk(x,k)

#define V2_kdiv(x,k) ((x)[0] = k/(x)[0], (x)[1] = k/(x)[1])
#define V3_kdiv(x,k) ((x)[0] = k/(x)[0], (x)[1] = k/(x)[1], (x)[2] = k/(x)[2])
#define V4_kdiv(x,k) ((x)[0] = k/(x)[0], (x)[1] = k/(x)[1], (x)[2] = k/(x)[2], (x)[3] = k/(x)[3])

#define I2_kdiv(x,k) V2_divk(x,k)
#define I3_kdiv(x,k) V3_divk(x,k)
#define I4_kdiv(x,k) V4_divk(x,k)
#define F2_kdiv(x,k) V2_divk(x,k)
#define F3_kdiv(x,k) V3_divk(x,k)
#define F4_kdiv(x,k) V4_divk(x,k)
#define D2_kdiv(x,k) V2_divk(x,k)
#define D3_kdiv(x,k) V3_divk(x,k)
#define D4_kdiv(x,k) V4_divk(x,k)

#define V2_mag(x) (sqrt((double)(x[0]*x[0]+x[1]*x[1])))
#define V3_mag(x) (sqrt((double)(x[0]*x[0]+x[1]*x[1]+x[2]*x[2])))
#define V4_mag(x) (sqrt((double)(x[0]*x[0]+x[1]*x[1]+x[2]*x[2]+x[3]*x[3])))

#define I2_mag(x) V2_mag(x)
#define I3_mag(x) V3_mag(x)
#define I4_mag(x) V4_mag(x)
#define F2_mag(x) V2_mag(x)
#define F3_mag(x) V3_mag(x)
#define F4_mag(x) V4_mag(x)
#define D2_mag(x) V2_mag(x)
#define D3_mag(x) V3_mag(x)
#define D4_mag(x) V4_mag(x)

#define V2_unit(x) 			\
{						 			\
double mag;						\
mag = (double)V2_mag(x);	\
V2_divk(x,mag);				\
}
#define V3_unit(x) 			\
{						 			\
double mag;						\
mag = (double)V3_mag(x);	\
V3_divk(x,mag);				\
}
#define V4_unit(x) 			\
{						 			\
double mag;						\
mag = (double)V4_mag(x);	\
V3_divk(x,mag);				\
}

#define I2_unit(x) V2_unit(x)
#define I3_unit(x) V3_unit(x)
#define I4_unit(x) V4_unit(x)
#define F2_unit(x) V2_unit(x)
#define F3_unit(x) V3_unit(x)
#define F4_unit(x) V4_unit(x)
#define D2_unit(x) V2_unit(x)
#define D3_unit(x) V3_unit(x)
#define D4_unit(x) V4_unit(x)

//-------------------------------------------------------------------------
//-------------------------------------------------------------------------

#define Pi 			3.14159265359
#define TwoPi 		6.28318530718
#define HalfPi  	1.570796326795

#define L 0   // Left
#define R 1   // Right

#define Min(a, b) ((a<b) ? a : b)
#define Max(a, b) ((a>b) ? a : b)
#define Sign(a) ((a<0.0) ? -1.0 : 1.0)
#define PiBound(a) ((a<-Pi) ? a+TwoPi : ((a>Pi) ? a-TwoPi : a))

#define XAXISDIR 1.0
#define YAXISDIR 1.0
#define ZAXISDIR 1.0

#define RAD_TO_DEG 57.29577951308
#define DEG_TO_RAD 0.01745329251994

#define intxLP_MAXVERTS    10
#define Max3Index( xx,yy,zz )   ((xx>yy)?(xx>zz?0:2):(yy>zz?1:2))
#define Min3Index( xx,yy,zz )   ((xx<yy)?(xx<zz?0:2):(yy<zz?1:2))
#define Max3( xx,yy,zz )   ((xx>yy)?(xx>zz?xx:zz):(yy>zz?yy:zz))
#define Min3( xx,yy,zz )   ((xx<yy)?(xx<zz?xx:zz):(yy<zz?yy:zz))

#define FALSE 0
#define TRUE  1

#define RENDER_NORMAL   	   	0x01
#define RENDER_BOUNDINGBOX    	0x02
#define RENDER_BOTHSIDES    		0x08
#define RENDER_FOG   	 			0x10
#define RENDER_MATERIAL  	   	0x20
#define RENDER_DISPLAYLIST       0x40

#ifdef _VRTYPES_OBJS
I2 zeroI2 = {0,0};
I3 zeroI3 = {0,0,0};
I4 zeroI4 = {0,0,0,0};
F2 zeroF2 = {0.0f,0.0f};
F3 zeroF3 = {0.0f,0.0f,0.0f};
F4 zeroF4 = {0.0f,0.0f,0.0f,0.0f};
D2 zeroD2 = {0.0,0.0};
D3 zeroD3 = {0.0,0.0,0.0};
D4 zeroD4 = {0.0,0.0,0.0,0.0};

F4 red    = {1.0f, 0.0f, 0.0f, 1.0f};
F4 green  = {0.0f, 1.0f, 0.0f, 1.0f};
F4 blue   = {0.0f, 0.0f, 1.0f, 1.0f};
F4 black  = {0.0f, 0.0f, 0.0f, 1.0f};
F4 white  = {1.0f, 1.0f, 1.0f, 1.0f};
F4 magenta= {1.0f, 0.0f, 1.0f, 1.0f};
F4 yellow = {1.0f, 1.0f, 0.0f, 1.0f};
F4 cyan   = {0.0f, 1.0f, 1.0f, 1.0f};

#else
extern I2 zeroI2;
extern I3 zeroI3;
extern I4 zeroI4;
extern F2 zeroF2;
extern F3 zeroF3;
extern F4 zeroF4;
extern D2 zeroD2;
extern D3 zeroD3;
extern D4 zeroD4;
extern F4 red;
extern F4 green;
extern F4 blue;
extern F4 black;
extern F4 white;
extern F4 magenta;
extern F4 yellow;
extern F4 cyan;
#endif

#ifdef AMBER_FLOAT_LIB
#define V3 F3
#define V4 F4
#define V3_copy(dest, src) F3_copy(dest,src)
#define V4_copy(dest, src) F4_copy(dest,src)
#define Vres float
#define zeroV3 zeroF3
#define zeroV4 zeroF4
#elif AMBER_DOUBLE_LIB
#define V3 D3
#define V4 D4
#define V3_copy(dest, src) D3_copy(dest,src)
#define V4_copy(dest, src) D4_copy(dest,src)
#define Vres double
#define zeroV3 zeroD3
#define zeroV4 zeroD4
#endif


#endif
